<?php

namespace Barn2\Plugin\WC_Multiple_Email_Recipients\Admin;

use Barn2\WMER_Lib\Registerable,
	Barn2\WMER_Lib\Service,
	Barn2\WMER_Lib\Service_Container,
	Barn2\WMER_Lib\Plugin\Licensed_Plugin,
	Barn2\WMER_Lib\Plugin\Admin\Admin_Links,
	Barn2\Plugin\WC_Multiple_Email_Recipients\Admin\Wizard\Setup_Wizard;

/**
 * General admin functions for WooCommerce Multiple Email Recipients.
 *
 * @package   Barn2/woocommerce-multiple-email-recipients
 * @author    Barn2 Plugins <info@barn2.co.uk>
 * @license   GPL-3.0
 * @copyright Barn2 Media Ltd
 */
class Admin_Controller implements Registerable, Service {

	use Service_Container;

	private $plugin;

	/**
	 * @var Admin_Links
	 */
	private $admin_links;

	public function __construct( Licensed_Plugin $plugin ) {
		$this->plugin = $plugin;
	}

	public function get_services() {
		$services = [
			'admin_links'   => new Admin_Links( $this->plugin ),
			'settings_page' => new Settings_Page( $this->plugin ),
			'wizard'        => new Setup_Wizard( $this->plugin ),
		];

		if ( $this->plugin->get_license()->is_valid() ) {
			// TODO: conditionally add services that are available to registered users only
		}

		return $services;
	}

	public function register() {
		$this->register_services();

		add_action( 'admin_enqueue_scripts', [ $this, 'admin_enqueue_scripts' ] );
	}

	public function admin_enqueue_scripts( $hook ) {
		global $current_tab, $current_section;

		$script_handle = '';
		$script_path   = '';
		$script_params = [];

		if ( 'products' === $current_tab && 'multiple-email-recipients' === $current_section ) {
			$script_handle = 'wmer_settings';
			$script_path   = '/assets/js/admin/wmer-settings.min.js';
			$script_params = [
				'i18n' => [
					'email_label_placeholder' => __( 'Email address', 'woocommerce-multiple-email-recipients' ),
					'warning_reduce_emails'   => __( 'Warning! If you reduce the number of additional email address fields then any customer data stored in these fields will become inactive. Notifications will only be sent to the remaining email addresses.', 'woocommerce-multiple-email-recipients' ),
				]
			];
		}

		if ( 'email' === $current_tab && $current_section ) {
			$script_handle = 'wmer_settings';
			$script_path   = '/assets/js/admin/email-settings.min.js';
			$script_params = [
				'i18n'   => [
					'warning_remove_recipient' => __( 'Are you sure you want to remove these additional recipients?', 'woocommerce-multiple-email-recipients' ),
					'add_recipient'            => __( 'Add recipient', 'woocommerce-multiple-email-recipients' ),
					'recipient_tip'            => __( 'Enter additional recipients (comma separated) for this email, and optionally select which products and/or categories they will receive it for.', 'woocommerce-multiple-email-recipients' ),
				],
				'emails' => $this->get_email_list(),
			];

			wp_enqueue_style( 'wmer_email_style', $this->plugin->get_dir_url() . '/assets/css/admin/email-settings.min.css', [], $this->plugin->get_version() );
		}

		if ( $script_handle && $script_path ) {
			wp_enqueue_script( $script_handle, $this->plugin->get_dir_url() . $script_path, [ 'jquery' ], $this->plugin->get_version(), true );
		}

		if ( ! empty( $script_params ) ) {
			wp_add_inline_script(
				$script_handle,
				sprintf( 'var wmer_params = %s;', wp_json_encode( $script_params ) ),
				'before'
			);
		}
	}

	private function get_email_list() {
		$emails_service = $this->plugin->get_service( 'emails_service' );

		if ( is_null( $emails_service ) ) {
			return [];
		}

		$emails = $emails_service->get_emails();

		return array_combine(
			array_map( 'strtolower', array_keys( $emails ) ),
			array_map(
				function( $v ) {
					return [
						'id'          => $v->id,
						'is_customer' => $v->is_customer_email()
					];
				},
				$emails
			)
		);
	}

}
