<?php

namespace Barn2\Plugin\WC_Multiple_Email_Recipients;

use Barn2\Plugin\WC_Multiple_Email_Recipients\Email_Providers\Admin_User_Profile_Provider;
use Barn2\Plugin\WC_Multiple_Email_Recipients\Email_Providers\Checkout_Provider;
use Barn2\WMER_Lib\Admin\Notices;
use Barn2\WMER_Lib\Registerable,
	Barn2\WMER_Lib\Translatable,
	Barn2\WMER_Lib\Service_Provider,
	Barn2\WMER_Lib\Plugin\Premium_Plugin,
	Barn2\WMER_Lib\Plugin\Licensed_Plugin,
	Barn2\WMER_Lib\Util as Lib_Util;

use function add_action;
use function add_filter;

/**
 * The main plugin class. Responsible for setting up to core plugin services.
 *
 * @package   Barn2/woocommerce-multiple-email-recipients
 * @author    Barn2 Plugins <info@barn2.co.uk>
 * @license   GPL-3.0
 * @copyright Barn2 Media Ltd
 */
class Plugin extends Premium_Plugin implements Licensed_Plugin, Registerable, Translatable, Service_Provider {

	const NAME    = 'WooCommerce Multiple Email Recipients';
	const ITEM_ID = 220192;

	private $services = [];

	/**
	 * @var Fields_Manager
	 */
	private $fields_manager;

	/**
	 * Constructs and initializes the main plugin class.
	 *
	 * @param string $file The main plugin file.
	 * @param string $version The current plugin version.
	 */
	public function __construct( $file = null, $version = '1.0' ) {
		parent::__construct(
			[
				'name'               => self::NAME,
				'item_id'            => self::ITEM_ID,
				'version'            => $version,
				'file'               => $file,
				'is_woocommerce'     => true,
				'settings_path'      => 'admin.php?page=wc-settings&tab=products&section=multiple-email-recipients',
				'documentation_path' => '/kb-categories/multiple-email-recipients-kb/',
			]
		);

		$this->fields_manager = new Fields_Manager();
	}

	/**
	 * @return Fields_Manager
	 */
	public function get_fields_manager() {
		return $this->fields_manager;
	}

	/**
	 * Registers the plugin with WordPress.
	 */
	public function register() {
		parent::register();

		add_action( 'init', [ $this, 'load_services' ], 15 );
		add_action( 'init', [ $this, 'load_textdomain' ] );
		add_filter( 'plugin_row_meta', [ $this, 'add_view_details_link' ], 10, 2 );
	}

	public function load_services() {

		// Don't load anything if WooCommerce not active.
		if ( ! Lib_Util::is_woocommerce_active() ) {
			$this->add_missing_woocommerce_notice();

			return;
		}

		// Create the admin service.
		if ( Lib_Util::is_admin() ) {
			$this->services['admin'] = new Admin\Admin_Controller( $this, $this->get_license_setting() );
		}

		// Create core services if license is valid.
		if ( $this->get_license()->is_valid() ) {

			// if ( $this->get_fields_manager()->need_to_provide_additional_emails() ) {
				$this->services['fields_service'] = new Fields_Service( $this );
				$this->services['emails_service'] = new Emails_Service( $this );
			// }
		}

		Lib_Util::register_services( $this->services );
	}

	public function get_service( $id ) {
		return isset( $this->services[ $id ] ) ? $this->services[ $id ] : null;
	}

	public function get_services() {
		return $this->services;
	}

	public function load_textdomain() {
		\load_plugin_textdomain( 'woocommerce-multiple-email-recipients', false, $this->get_slug() . '/languages' );
	}

	private function add_missing_woocommerce_notice() {
		if ( Lib_Util::is_admin() ) {
			$admin_notice = new Notices();
			$admin_notice->add(
				'wps_woocommerce_missing',
				'',
				\sprintf( __( 'Please %1$sinstall WooCommerce%2$s in order to use WooCommerce Multiple Email Recipients.', 'woocommerce-multiple-email-recipients' ), Lib_Util::format_link_open( 'https://woocommerce.com/', true ), '</a>' ),
				[
					'type'       => 'error',
					'capability' => 'install_plugins'
				]
			);
			$admin_notice->boot();
		}
	}

	public function add_view_details_link( $links, $file ) {

		if ( $file !== $this->get_basename() ) {
			return $links;
		}

		$view_details_url = 'https://barn2.co.uk/wordpress-plugins/woocommerce-multiple-email--recipients';

		\array_push( $links, \sprintf( '<a href="%1$s" target="_blank">%2$s</a>', \esc_url( $view_details_url ), __( 'View details', 'woocommerce-multiple-email-recipients' ) ) );

		return $links;
	}

}
